import { Component } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { ToastService } from 'src/app/service/toast.service';
import { FormPageBase } from 'src/app/core/base/form-page-base';
import {
  UserSpace,
  UserSpaceCreateInput,
  UserSpaceUpdateInput,
} from 'src/app/model/entities/user-space';
import { UserSpacesDataService } from 'src/app/modules/admin/user-spaces/user-spaces.data.service';

@Component({
  selector: 'app-user-space-form-page',
  templateUrl: './user-space-form-page.component.html',
  styleUrl: './user-space-form-page.component.scss',
})
export class UserSpaceFormPageComponent extends FormPageBase<
  UserSpace,
  UserSpaceCreateInput,
  UserSpaceUpdateInput,
  UserSpacesDataService
> {
  constructor(private toast: ToastService) {
    super(undefined, '../..');
    if (!this.nodeId) {
      this.node = this.new();
      this.setForm(this.node);

      return;
    }

    this.dataService.loadById(this.nodeId).subscribe(node => {
      this.node = node;
      this.setForm(this.node);
    });
  }

  new(): UserSpace {
    return {} as UserSpace;
  }

  buildForm() {
    this.form = new FormGroup({
      id: new FormControl<number | undefined>(undefined),
      name: new FormControl<string | undefined>(undefined, [
        Validators.required,
      ]),
    });
    this.form.controls['id'].disable();
  }

  setForm(node?: UserSpace) {
    this.form.controls['id'].setValue(node?.id);
    this.form.controls['name'].setValue(node?.name);

    this.spinner.hide();
  }

  getCreateInput(): UserSpaceCreateInput {
    return {
      name: this.form.controls['name'].value ?? undefined,
    };
  }

  getUpdateInput(): UserSpaceUpdateInput {
    if (!this.node?.id) {
      throw new Error('Node id is missing');
    }

    return {
      id: this.form.controls['id'].value,
      name: this.form.controls['name'].value ?? undefined,
    };
  }

  create(object: UserSpaceCreateInput): void {
    this.dataService.create(object).subscribe(() => {
      this.spinner.hide();
      this.toast.success('action.created');
      this.close();
    });
  }

  update(object: UserSpaceUpdateInput): void {
    this.dataService.update(object).subscribe(() => {
      this.spinner.hide();
      this.toast.success('action.created');
      this.close();
    });
  }
}
